/**
* Filename: RemoteStoreTest.java (in org.openbandy.example.test)
* This file is part of the OpenBandy project.
*
* www.openbandy.org
*/

package org.openbandy.example.test;

import org.openbandy.example.EuropeanSportscar;
import org.openbandy.example.remotestore.RemoteStoreProtocol;
import org.openbandy.example.test.TestDataGenerator;
import org.openbandy.io.Message;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.pref.Preferences;
import org.openbandy.pref.UnknownPreferenceException;
import org.openbandy.service.LogService;
import org.openbandy.service.MessageService;
import org.openbandy.test.Test;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class RemoteStoreTest extends Test {

	/* Constructor */
	public RemoteStoreTest() {
		super("Remote Store");
	}
	public void run() {
		testStarted();
		
		String username = "";
		
		/* read user name from preferences */
		try {
			username = Preferences.getValue("Username");
		} catch (UnknownPreferenceException upe) {
			LogService.error(this, "Preference 'Username' not known", upe);
			username = "unknown";
		}
		
		try {
			/* read server host address from preferences */
			String host = Preferences.getValue("Server Address");
			
			/* get a dummy object to store on the server */
			EuropeanSportscar europeanSportscar = TestDataGenerator.createDummyEuropeanSportscar();
			
			/* create a request*/
			RemoteStoreProtocol protocol = new RemoteStoreProtocol();
			Message request = protocol.createMessage(username, europeanSportscar);
			
			/* send it to the server */
			Message respsone = MessageService.sendMessage(host, protocol, request);
			
			/* the test failed if we do not get a response from the server */
			if(respsone == null){
				testsScreen.setTestFailed(this);
			}
			else {
				try{
					/* read the object again from the response */
					EuropeanSportscar databaseSerializedCar = (EuropeanSportscar)respsone.getContent();
					
					/* check if the two car objects are equal */
					if(databaseSerializedCar.isEqual(europeanSportscar)){
						
						/* check if the returned object has a valid database id */
						if(SerializationIdentifiers.isDatabaseSerializableId(databaseSerializedCar.getIdValue())){
							testsScreen.setTestPassed(this);
						}
						else {
							LogService.warn(this, "Objects does not have a valid databse id");
							testsScreen.setTestFailed(this);
						}
					}
					else{
						LogService.warn(this, "Objects are not equal");
						testsScreen.setTestFailed(this);
					}
				}
				catch(ClassCastException cce){
					LogService.error(this, "Response is not a 'EuropeanSportscar' object", cce);
					testsScreen.setTestFailed(this);
				}
			}			
		} catch (UnknownPreferenceException upe) {
			LogService.error(this, "Preference 'Server Address' not known", upe);
			testsScreen.setTestFailed(this);
		}
		
		testFinished();
	}
}
