/**
* Filename: RmsSerializerImpl.java (in org.openbandy.io.rms)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/

package org.openbandy.io.rms;

import java.util.Enumeration;
import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializableVector;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.io.storage.ObjectEntry;
import org.openbandy.io.storage.ObjectIdString;
import org.openbandy.io.storage.Storage;
import org.openbandy.io.storage.StorageSerializationBuffer;
import org.openbandy.service.LogService;
import org.openbandy.util.StringUtil;


/**
*  TODO describe purpose and usage
*  
* TODO all diese iterationen ber irgend einen vector knnte man 
* noch optimieren (hashtabellen/bitmaps etc). bentige zuviele!
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class RmsSerializerImpl implements RmsSerializer {
	
	/* ********* Service variables *********** */
	
	private static boolean isSetup = false;
	
	
	/* **** Constructor **** */
	
	public RmsSerializerImpl(){
		/* access the rms storage in order to init it (TODO could be made better..) */
		RmsStorage.getInstance();
		isSetup = true;
	}
    

    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageSerializer#store(org.openbandy.io.Serializable)
     */
    public synchronized void store(Serializable serializable) {

    	if(serializable != null){

    		/* reusable objectEntry variable */
    		ObjectEntry objectEntry;

    		/* initialize store */
    		StorageSerializationBuffer serializationBuffer = new StorageSerializationBuffer();

    		/* serialize */
    		serializeObject(serializable, serializationBuffer);

    		/* reverse order in tables */
    		Vector objectEntries = serializationBuffer.getAllObjectEntriesInReverseOrder();

    		Enumeration storeTables = objectEntries.elements();
    		while(storeTables.hasMoreElements()){

    			objectEntry = (ObjectEntry) storeTables.nextElement();
    			Serializable owner = objectEntry.getOwner();

    			/* for any child objects, insert created keys */
    			updateReferences(objectEntry);

    			RmsConnection objectConn;
    			if(canUpdate(owner, serializationBuffer)){
    				/* already in database, update */
    				objectConn = RmsStorage.getConnectionForObject(owner);
    				if(objectConn != null){
    					objectConn.executeUPDATE(objectEntry);
    				}
    			}else{
    				/* insert, and retrieve newly generated key */
    				objectConn = RmsStorage.getInstance().addObject(owner);                  
    				int newKey = objectConn.executeINSERT(objectEntry);

    				/* update my own key */
    				if(serializationBuffer.isSuperCopy(owner)){
    					Serializable subType = serializationBuffer.getSubType(owner);
    					subType.setIdValue(newKey);
    					LogService.debug(this, "Generated new rms id : "+objectEntry.getOwner().getIdName()+"="+newKey);
    				}else if(serializationBuffer.containsSubType(owner)){
    					/* do nothing, key already set */
    				}else{
    					owner.setIdValue(newKey);
    					LogService.debug(this, "Generated new rms id : "+objectEntry.getOwner().getIdName()+"="+newKey);

    				}

    			}

    			/* do not close the connection, as it can be reused */
    		}
    	}
    	LogService.info(this, "Stored object: "+serializable.getLegibleString());
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageSerializer#load(org.openbandy.io.Serializable, int)
     */
    public boolean load(Serializable serializable, int id) {
    	
		/* create and setup new proxy object */
		StorageSerializationBuffer serializationBuffer = new StorageSerializationBuffer();
    	
        if(serializable != null){
            
            /* set id */
            serializable.setIdValue(id);
            
            /* deserialize */
            StorageSerializationBuffer storageSerializationBuffer = (StorageSerializationBuffer)serializationBuffer;
            return deserializeObjectFromRms(serializable, storageSerializationBuffer);
        }
        return false;
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.Serializer#serializeObject(org.openbandy.io.Serializable, org.openbandy.io.SerializationBuffer)
     */
    public void serializeObject(Serializable serializable, SerializationBuffer serializationBuffer) {
        if(serializable != null){
        	
        	/* handle vector */
            if(serializable instanceof SerializableVector){
                ((SerializableVector)serializable).prepareStorageSerialization(RmsStorage.getInstance(), this);
            }
        	
            /* create new RmsValueContainer */
        	ObjectEntry objectEntry = new ObjectEntry(serializable);
            RmsValueContainer container = new RmsValueContainer(objectEntry);
            
            /* add container to buffer */
            serializationBuffer.addContainer(container);
            
            /* serialize object */
            serializable.serialize(this, serializationBuffer);
        }
    }
    
    /**
     * Deserialize a serializable object, get the correct id from the parent containter.
     */
    /* (non-Javadoc)
     * @see org.openbandy.io.Serializer#deserializeObject(org.openbandy.io.Serializable, org.openbandy.io.SerializationBuffer)
     */
    public void deserializeObject(Serializable serializable, SerializationBuffer serializationBuffer) {
        if(serializable != null){
            try{
                RmsValueContainer parentRmsContainer = (RmsValueContainer)serializationBuffer.container();
                
                /* set id */
                int myId = parentRmsContainer.getIdForObject(serializable);
                serializable.setIdValue(myId);
                
                /* deserialize */
                StorageSerializationBuffer storageSerializationBuffer = (StorageSerializationBuffer)serializationBuffer;
                deserializeObjectFromRms(serializable, storageSerializationBuffer);
                
            }catch(ClassCastException cce){
            	LogService.error(this, cce.getMessage(), cce);
            }
        }
    }
    
    /**
     * Deserialize an object from its object connn
     * 
     * @param serializable
     * @param objectConn
     * @return
     */
    public boolean deserializeObject(Serializable serializable, RmsConnection objectConn){
    	
    	/* create and setup new proxy object */
		StorageSerializationBuffer serializationBuffer = new StorageSerializationBuffer();
    	
        if(serializable != null){
            /* set the ojbects id */
            String objectsShortName = StringUtil.getShortClassName(serializable);
            String recordStoreName = objectConn.getRecordStoreName();
            
            String idString = recordStoreName.substring(objectsShortName.length());
            serializable.setIdValue(Integer.parseInt(idString));
            
            return deserializeObjectFromConnection(objectConn, serializable, serializationBuffer);
        }
        return false;
    }
    
    
    /* ********************** Helper Methods ******************* */
    
    /**
     * Deserialize a serialzable object that has the correct id set
     * 
     * @param serializable
     * @return
     */
    private boolean deserializeObjectFromRms(Serializable serializable, StorageSerializationBuffer serializationBuffer){
        /* get data from rms */
        if(SerializationIdentifiers.isValidId(serializable.getIdValue())){
            RmsConnection objectConn = RmsStorage.getConnectionForObject(serializable);
            if(objectConn != null){
                return deserializeObjectFromConnection(objectConn, serializable, serializationBuffer);
            }
        }
        return false;
    }
    
    private boolean deserializeObjectFromConnection(RmsConnection objectConn, Serializable serializable, StorageSerializationBuffer serializationBuffer){
        if( (objectConn != null) && (serializable != null) ){
            if(SerializationIdentifiers.isValidId(serializable.getIdValue())){
                /* create value container */
                RmsValueContainer container = new RmsValueContainer(((RmsConnection)objectConn).getRecordStore(), serializable);
                
                /* handle vector */
                if(serializable instanceof SerializableVector){
                    ((SerializableVector)serializable).prepareStorageDeserialization(container);
                }
                
                /* add container to buffer */
                serializationBuffer.addContainer(container);
                
                /* deserialize object */
                serializable.deserialize(this, serializationBuffer);
                serializationBuffer.removeContainer(container);
                return true;
            }else{
            	LogService.warn(this, "No object in rms with "+serializable.getIdName()+"="+serializable.getIdValue());
            }
        }
        return false;
    }
    
    private boolean canUpdate(Serializable serializable, StorageSerializationBuffer serializationBuffer){
        /* if serializable is a copy of an objects super type, check the subtype */
        if(serializationBuffer.isSuperCopy(serializable)){
            Serializable subType = serializationBuffer.getSubType(serializable);
            serializable.setIdValue(subType.getIdValue());
            
            serializable = subType;
        }
        
        if(SerializationIdentifiers.isValidId(serializable.getIdValue())){
            return (RmsStorage.getInstance().existsInStorage(serializable) > 0);
        }else{
            return false;
        }  
    }
    
    /**
     * TODO comment!
     *
     * @param objectEntry
     */
    public static void updateReferences(ObjectEntry objectEntry){
        
        Serializable owner = objectEntry.getOwner();
        
        /* handle vector */
        if(owner instanceof SerializableVector){
            SerializableVector vector = (SerializableVector)owner;
            if(vector.size()>0){
                
                /* store contained object ids */
                Enumeration containedObjects = vector.elements();
                ObjectIdString containedObjectIds = new ObjectIdString();
                
                while(containedObjects.hasMoreElements()){
                    /* add id of every contained object to the object id string */
                    Serializable containedObject = (Serializable)containedObjects.nextElement();
                    containedObjectIds.add(containedObject.getIdValue());
                }
                objectEntry.add(SerializableVector.CONTAINED_OBJS_IDS, containedObjectIds.getString());
            }
        }else{
            Vector referencedObjects = new Vector();
            owner.getReferences(referencedObjects);
            
            Enumeration elements = referencedObjects.elements();
            while(elements.hasMoreElements()){
                Serializable referencedObject = (Serializable)elements.nextElement();
                if(referencedObject != null){
                    
                    /* determine the name of the foreign key */
                    String idName = "";
                    if(referencedObject instanceof SerializableVector){
                        idName = ((SerializableVector)referencedObject).getSerializationName();
                    }else{
                        idName = referencedObject.getIdName();
                    }
                    objectEntry.update(idName, Integer.toString(referencedObject.getIdValue()));
                }
            }
        }
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    public boolean isSetup(){
    	return isSetup;
    }


	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.StorageSerializer#getStorage()
	 */
	public Storage getStorage() {
		return RmsStorage.getInstance();
	}
    
}
