/**
 * Filename: StreamValueContainer.java (in org.openbandy.io.stream)
 * This is part of the OpenBandy project.
 * 
 * www.openbandy.org
 */

package org.openbandy.io.stream;

import java.io.IOException;

import org.kxml.Attribute;
import org.kxml.io.AbstractXmlWriter;
import org.kxml.parser.ParseEvent;
import org.openbandy.io.ValueContainer;
import org.openbandy.service.LogService;
import org.openbandy.test.Assertion;

/**
 * This value container is a wrapper for WbxmlParser and WbmXmlWriter.
 *
 * <br><br>
 * (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public class StreamValueContainer extends ValueContainer {


	/* internal data structure for store purposes */
	private AbstractXmlWriter writer;

	/* internal data structure for load purposes */
	private ParseEvent event;

	
	/* **** constructors **** */
	
	public StreamValueContainer(AbstractXmlWriter writer){
		this.event = null;
		this.writer = writer;
	}
	
	public StreamValueContainer(ParseEvent event){
		this.event = event;
		this.writer = null;
	}


	/* **** add **** */

	public void addString(String name, String value){
		if( Assertion.notNull(writer) && Assertion.notNull(value) ){
			try{
				writer.attribute(name, value);
			}catch(IOException ioe){
				LogService.error(this, ioe.getMessage(), ioe);
			}catch(RuntimeException re){
				//TODO remove dev code
				re.printStackTrace();
			}
		}   
	}

	
	/* **** get **** */
	
	public String getString(String name) {
		if(Assertion.notNull(event)){
			Attribute attr = event.getAttribute(name);
			if(attr != null){
				return attr.getValue();
			}else{
				LogService.warn(this, "The element '"+event.getName()+"' does not have an attribute named '"+name+"'");
			}
		}
		return "";
	}
}
