package org.openbandy.log;

/**
 * The LogLevel class defines the different log levels as integer values,
 * as strings and the colors that are used to print (i.e. draw) the log
 * messages. Additionaly, it provides some auxiliary conversion methods.
 * 
 * @version 1.2
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public final class LogLevel {

	/** The log levels */
	public static final int ERROR = 0;
	public static final int WARNING = 1;
	public static final int INFO = 2;
	public static final int DEBUG = 3;
	public static final int DEFAULT = INFO;

	public static final int NUMBER_OF_LEVELS = 4;

	//TODO schner kommentieren
	/* note: all strings have lenght 7 ! */
	private static final String STRING_ERROR = "Error  ";
	private static final String STRING_WARNING = "Warning";
	private static final String STRING_INFO = "Info   ";
	private static final String STRING_DEBUG = "Debug  ";

	//TODO comment
	public static final String[] levels = new String[NUMBER_OF_LEVELS];
	static{	
		levels[ERROR] = STRING_ERROR;
		levels[WARNING] = STRING_WARNING;
		levels[INFO] = STRING_INFO;
		levels[DEBUG] = STRING_DEBUG;
	}

	/** The message colors for the different log levels */
	private static final int COLOR_ERROR = 13893632;		// dark red
	private static final int COLOR_WARNING = 13143296;	// dark orange
	private static final int COLOR_INFO = 0;			// black
	private static final int COLOR_DEBUG = 5131854;	    // dark grey

	/** Alternativ Color Scheme */
//	private static final int COLOR_ERROR = 8781956;		// dark red
//	private static final int COLOR_WARNING = 8798720;	// dark orange
//	private static final int COLOR_INFO = 0;			// black
//	private static final int COLOR_DEBUG = 17030;	    // dark blue


//	TODO comment
	public static int[] colors = new int[NUMBER_OF_LEVELS];
	static{	
		colors[ERROR] = COLOR_ERROR;
		colors[WARNING] = COLOR_WARNING;
		colors[INFO] = COLOR_INFO;
		colors[DEBUG] = COLOR_DEBUG;
	}
	
	/**
	 * Get the int value of the log level which is
	 * represented as a string. (Ingore case)
	 *
	 * @param logLevelString The log level as a string, e.g. Info or Debug
	 * @return The corresponding int value (e.g. 2 for Info) if no match is found, DEFAULT will be returned
	 */
	public static int getLevelFromString(String logLevelString){
		if(logLevelString.equalsIgnoreCase(STRING_ERROR.trim())){
			return ERROR;
		}
		if(logLevelString.equalsIgnoreCase(STRING_WARNING.trim())){
			return WARNING;
		}
		if(logLevelString.equalsIgnoreCase(STRING_INFO.trim())){
			return INFO;
		}
		if(logLevelString.equalsIgnoreCase(STRING_DEBUG.trim())){
			return DEBUG;
		}
		return DEFAULT;
	}


}
