package org.openbandy.log;

import java.io.IOException;

import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;

import org.openbandy.service.LogService;
import org.openbandy.ui.BandyDisplayable;



/**
 * This Form is used to set the level of logging
 * (i.e. which kind of log messages are logged).
 * 
 * @version 1.0
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public class SelectLogLevelForm extends List implements BandyDisplayable, CommandListener{

	/* Commands */
	private Command cmdOk;
	private Command cmdCancel;

	/* Images */
	private Image imgChecked;
	private Image imgUnchecked;
	
	/** Reference to the MIDlets display */
	private Display display;
	
	/** Reference to the log canvas */
	private LogImpl logCanvas;

	/* ************ Constructor ***************** */

	public SelectLogLevelForm() {
		super("Log Level", Choice.IMPLICIT);

		/* compose the GUI */
		cmdOk = new Command("Ok", Command.OK, 1);
		cmdCancel = new Command("Cancel", Command.CANCEL, 1);
		addCommand(cmdOk);
		addCommand(cmdCancel);
		
		/* Ok is the default command */
		this.setSelectCommand(cmdOk);

		/* load images */
		try{
			imgChecked = Image.createImage("/img/checked.png");
			imgUnchecked = Image.createImage("/img/unchecked.png");
		}catch(IOException ioe){
			LogService.error(this, ioe.getMessage(), ioe);
		}

		/* make myself command listener */
		this.setCommandListener(this);
	}

	
	/* ******************** Methods for BandyDisplayable ************** */
	
	public void setDisplay(Display display, Displayable previousDisplayable) {
		this.display = display;
		try {
			this.logCanvas = (LogImpl)previousDisplayable;
			
			/* append levels, set images correctly */
			for(int i=0;i<LogLevel.levels.length;i++){
				if(i == logCanvas.config.getLogLevel()){
					append(LogLevel.levels[i], imgChecked);
				}else{
					append(LogLevel.levels[i], imgUnchecked);
				}
			}
		}
		catch(ClassCastException cce){
			LogService.error(this, "Must have LogCanvas as previous screen", cce);
		}
	}


	public void updateScreen() {
		/* nothing to do */
	}
	

	/* ******************** Methods for CommandListener ************** */
	
	public void commandAction(Command c, Displayable d) {
		if(c == cmdCancel){
			display.setCurrent(logCanvas);
		}
		else if(c == cmdOk){
			try {
				/* set new log level */
				logCanvas.config.setLogLevel(getSelectedIndex());
				
				/* reset log canvas and show it */
				logCanvas.resetCanvas();
				display.setCurrent(logCanvas);
			} catch (Exception e) {
				LogService.error(this, e.getMessage(), e);
			}
		}
	}
}
