/**
* Filename: MmsScanner.java (in org.openbandy.net.client)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.net.client;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Vector;

import javax.microedition.io.Connector;
import javax.wireless.messaging.MessageConnection;
import javax.wireless.messaging.MessagePart;
import javax.wireless.messaging.MultipartMessage;

import org.openbandy.event.EventSource;
import org.openbandy.event.EventSourceOwner;
import org.openbandy.io.Message;
import org.openbandy.io.stream.StreamSerializer;
import org.openbandy.io.stream.StreamSerializerImpl;
import org.openbandy.net.MessageBroker;
import org.openbandy.service.LogService;

/**
 * TODO describe purpose and usage
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class MmsScanner extends EventSource {
    
    /* stream serializer */
    private StreamSerializer serializer = null;
    
    /* client connection */
    private MessageConnection clientConnection = null;
    
    /* constructor */
    public MmsScanner(EventSourceOwner owner){
        super("ClientMmsScanner", owner);
        serializer = new StreamSerializerImpl();
    }
    
    /* ********** Methods for EventSource ************ */
    
    public Vector getProvidedEventTypes() {
        Vector providedEventTypes = new Vector();
        providedEventTypes.addElement("IncomingMessageEvent");
        return providedEventTypes;
    }
    
    public EventSource newlyInstantiate() {
        MmsScanner newInstance = new MmsScanner(this.owner);
        owner.setNewInstance(newInstance);
        return newInstance;
    }

    public void run() {
        
        /* variables */
        MultipartMessage multipartMessage = null;
        MessagePart bandyMessage = null;
        byte[] buffer = null;
        String address = "mms://:"+MessageBroker.MMS_PORT;
        String sender = "";
        
        /* start */
        running = true;
        
        while(running){
            clientConnection = null;

            try{
                /* open BLOCKING connection */
                clientConnection = (MessageConnection)Connector.open(address);
                LogService.debug(this, "Opened Connection "+address);
                
                /* wait for incoming binary sms messages */
                multipartMessage = (MultipartMessage)clientConnection.receive();
                sender = multipartMessage.getAddress();
                
                /* get message part */
                bandyMessage = multipartMessage.getMessagePart(MmsSender.BANDY_MMS_PAYLOAD);
                
                /* check mime type */
                if(bandyMessage.getMIMEType().equals(MmsSender.BANDY_MESSAGE_MIME)){
                    /* read binary mms message payload */
                    buffer = bandyMessage.getContent();
                    
                    /* deserialize message */
                    ByteArrayInputStream bais = new ByteArrayInputStream(buffer);
                    Message newMessage = (Message) serializer.getObjectFromXmlStream(bais);
                    
                    /* new message from server, put in queue and fire event */
                    LogService.info(this, "Message from "+sender+": "+newMessage.getLegibleString());
                    MessageScannerHelper.processNewMessage(newMessage);
                    
                }else{
                    LogService.warn(this, "Wrong MIME-type");
                }

                /* close connection */
                clientConnection.close();
                
            }catch(SecurityException se){
                /* the user did not allow to open port */
                LogService.warn(this, "Not authorized to open port for mms messages");
                stopEventSource();
            }catch(ClassCastException cce){
                LogService.error(this, "Incoming Message is not MultipartMessage", cce);
            }catch(IOException ioe){
                LogService.error(this, "Could not receive MMS from client", ioe);
            } 
        }
    }
    
    public void stopEventSource(String eventShortClassName) {
        consumedEvents.removeElement(eventShortClassName);
        
        if(consumedEvents.isEmpty()){
            this.running = false;
            LogService.debug(this, "Stoped EventSource : "+getName());
            
            if(clientConnection != null){
                try{
                    /* close connection */
                    clientConnection.close();
                }catch(IOException ioe){
                    LogService.error(this, "Could not close SMS connection", ioe);
                }
            }
        }
    }

    public void stopEventSource() {
        
        consumedEvents.removeAllElements();
        this.running = false;
        LogService.debug(this, "Stoped EventSource : "+getName());
        
        if(clientConnection != null){
            try{
                /* close connection */
                clientConnection.close();
            }catch(IOException ioe){
            	LogService.error(this, "Could not close SMS connection", ioe);
            }
        }
    }

}
