package org.openbandy.ui;

import java.io.IOException;
import java.util.Enumeration;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;

import org.openbandy.service.LogService;
import org.openbandy.test.Test;
import org.openbandy.util.DoubleHashtable;


/**
 * TODO describe purpose and usage
 * 
 * @version 1.0
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public class Tests extends List implements CommandListener, BandyDisplayable{
    
    /* Commands */
    private Command cmdBack;
    private Command cmdRun;
    private Command cmdRunAll;
    private Command cmdReset;
    
    /* Display reference */
    private Display display;
    
    /* Previous Screen */
    private Displayable previousDisplayable;
    
    /* Current list of tests */
    private DoubleHashtable tests = new DoubleHashtable();
    
    /* Images */
    private Image readyImage;
    private Image runningImage;
    private Image passedImage;
    private Image failedImage;
    
    /* count test runs */
    private int testCounter = 0;
    
    public Tests(){
        super("Tests", List.IMPLICIT);
        
        try{
            readyImage = Image.createImage("/img/unchecked.png");
            passedImage = Image.createImage("/img/checked.png");
            failedImage = Image.createImage("/img/stop.png");
            runningImage = Image.createImage("/img/running.png");
        }catch(IOException ioe){
            LogService.error(this, "Failed loading images", ioe);
        }
        
        /* create and add Commands */
        cmdBack = new Command("Back", Command.BACK, 1);
        cmdRun = new Command("Run selected test", Command.SCREEN, 1);
        cmdRunAll = new Command("Run all tests", Command.SCREEN, 1);
        cmdReset = new Command("Reset tests", Command.SCREEN, 2);
        addCommand(cmdBack);
        addCommand(cmdRun);
        addCommand(cmdRunAll);
        addCommand(cmdReset);
        
        this.setSelectCommand(cmdRun);
        
        /* make myself command listener */
        this.setCommandListener(this);
    }
    
    /**
     * TODO comment!
     *
     * @param test
     */
    public void addTest(Test test){
    	test.setTestsScreen(this);
        Integer index = new Integer(append(" "+test.getName(), readyImage));
        tests.put(index, test);
    }

    /**
     * TODO comment!
     *
     * @param test
     */
    public void setTestReady(Test test){
        Integer index = (Integer)tests.getKeyByValue(test);
        set(index.intValue(), test.getName(), readyImage);
    }
    
    /**
     * TODO comment!
     *
     * @param test
     */
    public void setTestRunning(Test test){
        Integer index = (Integer)tests.getKeyByValue(test);
        set(index.intValue(), test.getName(), runningImage);
    }
    
    /**
     * TODO comment!
     *
     * @param test
     */
    public void setTestPassed(Test test){
        Integer index = (Integer)tests.getKeyByValue(test);
        set(index.intValue(), test.getName(), passedImage);
    }
    
    /**
     * TODO comment!
     *
     * @param test
     */
    public void setTestFailed(Test test){
        Integer index = (Integer)tests.getKeyByValue(test);
        set(index.intValue(), test.getName(), failedImage);
    }
    
    private void runAllTests(){
    	if(testCounter > 0){
    		updateScreen();
    	}
    	Enumeration elements = tests.getValues();
    	while(elements.hasMoreElements()){
    		Test test = (Test)elements.nextElement();
    		setTestRunning(test);
    		test.start();
    	}
    	testCounter++;
    }

    private void runSelectedTest(){
    	Integer index = new Integer(this.getSelectedIndex());
    	Test selectedTest = (Test)tests.getValueByKey(index);
    	if(selectedTest != null){
    		setTestRunning(selectedTest);
    		selectedTest.start();
    	}
    }
    
    /* ************* Methods for BandyDisplayable **************** */
    
	public void setDisplay(Display display, Displayable previousDisplayable){
		this.display = display;
		this.previousDisplayable = previousDisplayable;
	}
    
    public void updateScreen() {
    	/* nothing to do */
    	/* could reset the status of all tests here if desired */
    }
    
    /* ************* Method for CommandListener **************** */
    
    public void commandAction(Command c, Displayable d) {
        if(c == cmdBack){
        	if( (display != null) && (previousDisplayable != null)) {
				display.setCurrent(previousDisplayable);
			}
			else {
				LogService.warn(this, "Previous display not set");
			}
        }else if(c == cmdRunAll){
            runAllTests();
        }else if(c == cmdRun){
        	runSelectedTest();
        }else if (c == cmdReset){
        	Enumeration elements = tests.getValues();
        	while(elements.hasMoreElements()){
        		Test test = (Test)elements.nextElement();
        		setTestReady(test);
        	}
        }
    }

}
